from earthdiagnostics import Utils, cdftools, TempFile
from earthdiagnostics.basins import Basins
from earthdiagnostics.models import Models
from autosubmit.config.log import Log
import numpy as np
import os
import shutil
import netCDF4


class Circulation(object):
    """
    Class containing all the diagnostics related to circulation
    """

    @staticmethod
    def moc(input_file, output_file):
        """
        Compute the MOC for oceanic basins
        Created in March 2012
        Author : vguemas@ic3.cat

        :param input_file:  input grid_V file namez
        :type input_file: str
        :param output_file: output file name (=> 2D, depth-y)
        :param output_file: str
        :return:
        """

        if os.path.exists(output_file):
            os.remove(output_file)
        temp = TempFile.get()
        temp2 = TempFile.get()
        Log.debug('Computing MOC')
        cdftools.run('cdfmoc', input=input_file, output=temp2)
        Utils.nco.ncks(input=input_file, output=temp2, options='-A -v lev')
        Log.debug('Reformating to netCDF-4')
        Utils.execute_shell_command(["nccopy", "-4", temp2, temp])
        os.remove(temp2)

        Log.debug('Reformatting variables')
        handler = Utils.cdo.openCdf(temp)

        handler.createDimension('basin', 5)
        handler.createVariable('basin', str, 'basin')
        handler.variables['basin'][:] = np.array([Basins.Global.fullname, Basins.Atlantic.fullname,
                                                  Basins.Pacific.fullname, Basins.IndoPacific.fullname,
                                                  Basins.Indian.fullname], dtype=object)
        example = handler.variables['zomsfglo']
        # noinspection PyProtectedMember
        moc = handler.createVariable('msftmyz', example.datatype,
                                     ('time', 'lev', 'i', 'j', 'basin'),
                                     fill_value=example._FillValue)

        moc.units = example.units
        moc.add_offset = example.add_offset
        moc.scale_factor = example.scale_factor

        moc[:, :, :, :, 0] = handler.variables['zomsfglo'][:]
        moc[:, :, :, :, 1] = handler.variables['zomsfatl'][:]
        moc[:, :, :, :, 2] = handler.variables['zomsfpac'][:]
        moc[:, :, :, :, 3] = handler.variables['zomsfinp'][:]
        moc[:, :, :, :, 4] = handler.variables['zomsfind'][:]

        handler.close()

        Utils.nco.ncks(input=temp, output=temp,
                       options='-O -x -v zomsfglo,zomsfatl,zomsfpac,zomsfinp,zomsfind,zomsfinp0')
        Utils.setminmax(temp, 'msftmyz')

        Utils.move_file(temp, output_file)

    @staticmethod
    def area_moc(input_file, lat_min, lat_max, output_file, depth_min, depth_max, basin):
        """
        Compute an Atlantic MOC index by averaging the meridional overturning
        in a latitude band between 1km and 2km
        or any other index averaging the meridional overturning in
        a given basin and a given domain

        Created in March 2012         Author : vguemas@ic3.cat

        :param input_file: input moc file name
        :param lat_min: latitude min
        :param lat_max: latitude max
        :param output_file: output file name      ( => index )
        :param depth_min: depth min
        :param depth_max: depth max
        :param basin: basin
        :return:
        """
        nco = Utils.nco
        cdo = Utils.cdo
        temp = TempFile.get()
        temp2 = TempFile.get()

        shutil.copy(input_file, temp)

        handler = Utils.nco.openCdf(temp)
        if 'i' in handler.dimensions:
            handler.close()
            nco.ncwa(input=temp, output=temp, options='-O -a i')
            handler = Utils.cdo.openCdf(temp)

        basin_instance = Basins.parse(basin)
        if not basin_instance:
            raise Exception("Basin {0} not recognized".format(basin))

        basin_index = np.where(handler.variables['basin'][:] == basin_instance.fullname)
        lat_values = handler.variables['lat'][:]
        lat_type = handler.variables['lat'].dtype
        lat_units = handler.variables['lat'].units
        lat_long_name = handler.variables['lat'].long_name

        handler.close()

        if len(basin_index) == 0:
            raise Exception('Basin {0} not defined in file')
        basin_index = basin_index[0][0]
        nco.ncks(input=temp, output=temp, options='-O -d basin,{0}'.format(basin_index))
        # To remove basin dimension
        nco.ncwa(input=temp, output=temp, options='-O -a basin')

        nco.ncks(input=temp, output=temp, options='-O -v msftmyz,time,lev')

        handler = Utils.nco.openCdf(temp)
        handler.renameDimension('j', 'lat')
        lat_variable = handler.createVariable('lat', lat_type, 'lat')
        lat_variable[:] = lat_values[:]
        lat_variable.units = lat_units
        lat_variable.long_name = lat_long_name

        handler.close()

        nco.ncks(input=temp, output=temp2,
                 options='-O -d lev,{0:.1f},{1:.1f} -d lat,{2:.1f},{3:.1f}'.format(depth_min, depth_max,
                                                                                   lat_min, lat_max))

        cdo.vertmean(input=temp2, output=temp)
        nco.ncap2(input=temp, output=temp, options='-O -s "coslat[lat]=cos(lat[lat]*3.141592657/180.0)"')
        nco.ncwa(input=temp, output=temp, options='-w coslat -a lat')
        nco.ncks(input=temp, output=temp, options='-O -v msftmyz,time')
        Utils.move_file(temp, output_file)

    @staticmethod
    def max_moc(input_file, lat_min, lat_max, output_file, depth_min, depth_max, basin=None):
        """
        Compute an Atlantic MOC index by finding the maximum  of the annual
        mean meridional overturning in a latitude / depth region

        Created in March 2012         Author : vguemas@ic3.cat

        :param basin: basin
        :param input_file: input moc file name
        :param lat_min: latitude min
        :param lat_max: latitude max
        :param output_file: output file name
        :param depth_min: depth mean
        :param depth_max: depth max
        :return:
        """
        nco = Utils.nco
        temp = TempFile.get()
        temp2 = TempFile.get()
        if not basin:
            basin = Basins.Global
        basin = Basins.parse(basin)

        shutil.copy(input_file, temp)
        handler = Utils.cdo.openCdf(temp)
        if 'i' in handler.dimensions:
            nco.ncwa(input=temp, output=temp, options='-O -a i')
        else:
            shutil.copy(temp, temp)
        basin_index = np.where(handler.variables['basin'][:] == basin.fullname)
        if len(basin_index) == 0:
            raise Exception("Basin {1} is not defined in {0}", input_file, basin.fullname)
        if len(basin_index) == 0:
            raise Exception('Basin {0} not defined in file')
        basin_index = basin_index[0][0]
        handler.close()

        # Utils.rename_variable(temp, 'record', 'x')
        handler = Utils.nco.openCdf(temp)
        lat_values = handler.variables['lat'][:]
        lat_type = handler.variables['lat'].dtype
        lat_units = handler.variables['lat'].units
        lat_long_name = handler.variables['lat'].long_name
        handler.close()
        nco.ncks(input=temp, output=temp, options='-O -d basin,{0}'.format(basin_index))
        # To remove basin dimension
        nco.ncwa(input=temp, output=temp, options='-O -a basin')

        nco.ncks(input=temp, output=temp, options='-O -v msftmyz,time,lev,lev_bnds')

        handler = Utils.nco.openCdf(temp)
        handler.renameDimension('j', 'lat')
        lat_variable = handler.createVariable('lat', lat_type, 'lat')
        lat_variable[:] = lat_values[:]
        lat_variable.units = lat_units
        lat_variable.long_name = lat_long_name
        handler.close()

        nco.ncks(input=temp, output=temp2,
                 options='-O -d lev,{0:.1f},{1:.1f} -d lat,{2:.1f},{3:.1f}'.format(depth_min, depth_max,
                                                                                   lat_min, lat_max))
        Utils.cdo.yearmean(input=temp2, output=temp)
        handler = nco.openCdf(temp)

        moc = handler.variables['msftmyz'][:]

        maximum = np.amax(moc)
        max_index = np.unravel_index(np.argmax(moc), moc.shape)
        max_lev = handler.variables['lev'][max_index[1]]
        max_lat = handler.variables['lat'][max_index[2]]

        minimum = np.amin(moc)
        minimum_index = np.unravel_index(np.argmin(moc), moc.shape)
        min_lev = handler.variables['lev'][minimum_index[1]]
        min_lat = handler.variables['lat'][minimum_index[2]]

        handler.close()

        Log.info('Maximum {0} Sv, latitude {1} depth {2} m', maximum, max_lat, max_lev)
        Log.info('Minimum {0} Sv, latitude {1} depth {2} m', minimum, min_lat, min_lev)

        os.remove(temp)

        handler = netCDF4.Dataset(temp, 'w')

        var = handler.createVariable('msftmyzmax', float)
        var.long_name = 'Maximum_Overturing'
        var.units = 'Sverdrup'
        var.valid_min = -1000.
        var.valid_max = 1000.
        var[:] = maximum

        var = handler.createVariable('msftmyzmaxlat', float)
        var.long_name = 'Latitude_of_Maximum_Overturing'
        var.units = 'Degrees'
        var.valid_min = -90.
        var.valid_max = 90.
        var[:] = lat_max

        var = handler.createVariable('msftmyzmaxlev', float)
        var.long_name = 'Depth_of_Maximum_Overturing'
        var.units = 'Meters'
        var.valid_min = 0.
        var.valid_max = 10000.
        var[:] = max_lev

        var = handler.createVariable('msftmyzmin', float)
        var.long_name = 'Minimum_Overtuning'
        var.units = 'Sverdrup'
        var.valid_min = -1000.
        var.valid_max = 1000.
        var[:] = maximum

        var = handler.createVariable('msftmyzminlat', float)
        var.long_name = 'Latitude_of_Minimum_Overtuning'
        var.units = 'Degrees'
        var.valid_min = -90.
        var.valid_max = 90.
        var[:] = lat_max

        var = handler.createVariable('msftmyzminlev', float)
        var.long_name = 'Depth_of_Minimum_Overtuning'
        var.units = 'Meters'
        var.valid_min = 0.
        var.valid_max = 10000.
        var[:] = max_lev

        handler.close()
        Utils.move_file(temp, output_file)

    @staticmethod
    def psi(input_file_u, input_file_v, output_file):
        """
        Compute the barotropic stream function

        Created in March 2012         Author : vguemas@ic3.cat
        z
        :param input_file_u: input grid_U file name
        :rtype input_file_U: str
        :param input_file_v: input grid_V file name
        :rtype input_file_V: str
        :param output_file: output file name without nc extension  (=> 2D x-y)
        :rtype output_file: str
        """
        temp = TempFile.get()
        cdftools.run('cdfpsi', input=[input_file_u, input_file_v], output=temp, options='-full -mask -mean')
        Utils.rename_variable(temp, 'sobarstf', 'msftbarot')
        Utils.setminmax(temp, 'msftbarot')
        Utils.move_file(temp, output_file)

    # noinspection PyPep8Naming
    @staticmethod
    def gyres(input_psi, input_grid, output_file):
        """
        Compute the intensity of the subtropical and subpolar gyres

        Created in October 2013         Author : vguemas@ic3.cat

        :param input_psi: input psi file name
        :param input_grid: input grid
        :param output_file: output file name   ( => index )
        :return:
        """

        if input_grid in [Models.ECEARTH_2_3_O1L42, Models.ECEARTH_3_0_O1L46,
                          Models.NEMO_3_2_O1L42, Models.NEMO_3_3_O1L46,
                          Models.NEMOVAR_O1L42]:
            subpolNAtl = [230, 275, 215, 245]
            subpolNPac = [70, 145, 195, 235]
            subtropNPac = [45, 175, 165, 220]
            subtropSPac = [195, 275, 175, 225]
            subtropNAtl = [70, 205, 120, 145]
            subtropSAtl = [235, 300, 120, 145]
            subtropInd = [320, 30, 110, 180]
            ACC = [1, 361, 1, 65]

        elif input_grid in [Models.ECEARTH_3_0_O25L46, Models.ECEARTH_3_0_O25L75,
                            Models.GLORYS2_V1_O25L75]:
            raise Exception("Option gyres not available yet for {0}".format(input_grid))
        else:
            raise Exception("Input grid {0} not recognized".format(input_grid))

        temp = TempFile.get()
        temp2 = TempFile.get()
        Circulation.gyre(input_psi, subpolNAtl, temp2,  invert=True)
        Utils.rename_variable(temp2, 'msftbarot', 'subpolNAtl')

        Circulation.gyre(input_psi, subpolNPac, temp, invert=True)
        Utils.rename_variable(temp, 'msftbarot', 'subpolNPac')
        Utils.nco.ncks(input=temp, output=temp2, options='-A')

        Circulation.gyre(input_psi, subtropNPac, temp)
        Utils.rename_variable(temp, 'msftbarot', 'subtropNPac')
        Utils.nco.ncks(input=temp, output=temp2, options='-A')

        Circulation.gyre(input_psi, subtropSPac, temp)
        Utils.rename_variable(temp, 'msftbarot', 'subtropSPac')
        Utils.nco.ncks(input=temp, output=temp2, options='-A')

        Circulation.gyre(input_psi, subtropNAtl, temp)
        Utils.rename_variable(temp, 'msftbarot', 'subtropNAtl')
        Utils.nco.ncks(input=temp, output=temp2, options='-A')

        Circulation.gyre(input_psi, subtropSAtl, temp)
        Utils.rename_variable(temp, 'msftbarot', 'subtropSAtl')
        Utils.nco.ncks(input=temp, output=temp2, options='-A')

        Circulation.gyre(input_psi, subtropInd, temp)
        Utils.rename_variable(temp, 'msftbarot', 'subtropInd')
        Utils.nco.ncks(input=temp, output=temp2, options='-A')

        Circulation.gyre(input_psi, ACC, temp)
        Utils.rename_variable(temp, 'msftbarot', 'ACC')
        Utils.nco.ncks(input=temp, output=temp2, options='-A')

        Utils.move_file(temp2, output_file)

    @staticmethod
    def gyre(input_file, site, output_file, invert=False):
        """
        Compute the intensity of a given gyre

        Created in October 2013         Author : vguemas@ic3.cat

        :param input_file: input oce file name containing msftbarot
        :param site: site to calculate convection on
        :param output_file: output file name (=> index)
        :param invert: if True, multiplies result by -1
        :return:
        """
        temp = TempFile.get()
        if invert:
            Utils.cdo.fldmin(input='-selindexbox,{0[0]},{0[1]},{0[2]},{0[3]} {1}'.format(site, input_file),
                             output=temp)
            Utils.cdo.mulc(-1, input=temp, output=output_file)
        else:

            Utils.cdo.fldmax(input='-selindexbox,{0[0]},{0[1]},{0[2]},{0[3]} {1}'.format(site, input_file),
                             output=output_file)

    @staticmethod
    def convection_sites(input_file, input_grid, output_file):
        """
        Compute the intensity of convection in the four main convection sites

        Created in October 2013         Author : vguemas@ic3.cat

        :param input_file: input oce file name containing mlotst
        :param input_grid: input grid
        :param output_file: output file name (=> index)
        :return:
        """
        if input_grid in [Models.ECEARTH_2_3_O1L42, Models.ECEARTH_3_0_O1L46,
                          Models.NEMO_3_2_O1L42, Models.NEMO_3_3_O1L46,
                          Models.NEMOVAR_O1L42]:
            labrador = [255, 245, 215, 255]
            irminger = [245, 290, 215, 245]
            gin = [260, 310, 245, 291]
            wedell = [225, 280, 1, 50]

        elif input_grid in [Models.ECEARTH_3_0_O25L46, Models.ECEARTH_3_0_O25L75,
                            Models.GLORYS2_V1_O25L75]:
            raise Exception("Option convection not available yet for {0}".format(input_grid))
        else:
            raise Exception("Input grid {0} not recognized".format(input_grid))

        labrador_file = TempFile.get()
        Circulation.convection_site(input_file, labrador, labrador_file)

        irminger_file = TempFile.get()
        Circulation.convection_site(input_file, irminger, irminger_file)

        gin_file = TempFile.get()
        Circulation.convection_site(input_file, gin, gin_file)

        wedell_file = TempFile.get()
        Circulation.convection_site(input_file, wedell, wedell_file)

        nco = Utils.nco
        Utils.rename_variable(labrador_file, 'mlotst', 'mlotstLabrador')
        Utils.rename_variable(irminger_file, 'mlotst', 'mlotstIrminger')
        Utils.rename_variable(gin_file, 'mlotst', 'mlotstGIN')
        Utils.rename_variable(wedell_file, 'mlotst', 'mlotstWedell')

        nco.ncks(input=irminger_file, output=labrador_file, options='-A -v mlotstIrminger')
        nco.ncks(input=gin_file, output=labrador_file, options='-A -v mlotstGIN')
        nco.ncks(input=wedell_file, output=labrador_file, options='-A -v mlotstWedell')

        Utils.move_file(labrador_file, output_file)

    @staticmethod
    def convection_site(input_file, site, output_file):
        """
        Compute the intensity of convection at one site

        Created in October 2013         Author : vguemas@ic3.cat

        :param input_file: input oce file name containing somxl010
        :param site: site to calculate convection on
        :param output_file: output file name (=> index)
        :return:
        """

        Utils.cdo.fldmax(input='-selindexbox,{0[0]},{0[1]},{0[2]},{0[3]} {1}'.format(site, input_file),
                         output=output_file)
