from autosubmit.config.log import Log

from earthdiagnostics import Utils, TempFile, cdftools
import shutil
import os
import numpy as np
import threading


class General(object):
    """
    General diagnostics that can be used with different variables
    """

    @staticmethod
    def vertical_mean(input_file, output_file, variable, lev_min, lev_max):
        """
        Choose vertical level in ocean, or vertically average between
        2 or more  ocean levels

        Created in  February 2012      Author : vguemas@ic3.cat
        Modified (more generic, i.e. for any input var) in December 2014
        Author : eleftheria.exarchou@ic3.cat

        :param input_file: input file name
        :param output_file: output file name  (=> 2D field  )
        :param variable: variable name
        :param lev_min: upper depth of the layer (in level number)
        :param lev_max: lower depth of the layer (in level number. If same as the upper layer, then the script just
        selects this layer. If its different, a vertical integral (weighted) between upper and lower level is computed
        """

        temp = TempFile.get()
        temp2 = TempFile.get()

        shutil.copy(input_file, temp)

        handler = Utils.nco.openCdf(temp)
        lev_min = handler.variables['lev'][lev_min]
        lev_max = handler.variables['lev'][lev_max]

        cdftools.run('cdfvertmean', input=temp, output=temp2, options=[variable, 'T', lev_min, lev_max,
                                                                       '-full', '-debug'])
        os.remove(temp)
        Utils.setminmax(temp2, '{0}_vert_mean'.format(variable))
        Utils.rename_variable(temp2, '{0}_vert_mean'.format(variable),
                              '{0}mean{1}-{2}'.format(variable, lev_min, lev_max))
        Utils.move_file(temp2, output_file)

    @staticmethod
    def interpolate3d(input_file, output_file, variable, nemo_version):
        """
        3-dimensional conservative interpolation to the regular atmospheric grid

        Created in November 2012      Author : vguemas@ic3.cat

        :param nemo_version:
        :param input_file:
        :param output_file:
        :param variable:
        :return:
        """
        temp = TempFile.get()
        temp2 = TempFile.get()
        shutil.copy(input_file, temp)

        cdo = Utils.cdo
        nco = Utils.nco
        handler = cdo.openCdf(temp)

        threads = list()
        numthreads = 8
        for numthread in range(0, numthreads):
            t = threading.Thread(target=General._interpolate_level,
                                 args=(range(numthread, handler.dimensions['lev'].size, numthreads), temp, variable,
                                       nemo_version))
            threads.append(t)
            t.start()

        for t in threads:
            t.join()

        nco.ncrcat(input='tmp_01.nc', output=temp2,
                   options='-n {0},2,1 -v {1}'.format(handler.dimensions['lev'].size, variable))

        for lev in range(0, handler.dimensions['lev'].size):
            os.remove('tmp_{0:02d}.nc'.format(lev+1))
        handler = nco.openCdf(temp2)
        handler.renameDimension('record', 'lev')
        handler.close()
        nco.ncpdq(input=temp2, output=temp2, options='-O -h -a time,lev')
        nco.ncks(input=temp, output=temp2, options='-A -v lev')

        cdo.setgrid('t106grid', input=temp2, output=temp)
        if nemo_version[6:9] == '025':
            cdo.invertlatdata(input=temp, output=temp)
        Utils.move_file(temp, output_file)

    @staticmethod
    def _interpolate_level(levs, input_file, variable, nemo_version):
        for lev in levs:
            nco = Utils.nco
            temp2 = TempFile.get()
            nco.ncks(input=input_file, output=temp2, options='-O -d lev,{0} -v {1}'.format(lev, variable))
            nco.ncwa(input=temp2, output=temp2, options='-O -h -a lev')
            namelist_file = 'scrip_use_in{0}'.format(lev)
            scrip_use_in = open(namelist_file, 'w')
            scrip_use_in.writelines("&remap_inputs\n")
            scrip_use_in.writelines("   remap_wgt   = '/esnas/autosubmit/con_files/"
                                    "weigths/{0}/rmp_{0}_to_regular_lev{1}.nc'\n".format(nemo_version, lev+1))
            scrip_use_in.writelines("   infile      = '{0}'\n".format(temp2))
            scrip_use_in.writelines("   invertlat   = FALSE\n")
            scrip_use_in.writelines("   var         = '{0}'\n".format(variable))
            scrip_use_in.writelines("   fromregular = FALSE\n")
            scrip_use_in.writelines("   outfile     = '{0}'\n".format(temp2))
            scrip_use_in.writelines("/\n")
            scrip_use_in.close()
            Utils.execute_shell_command('/home/Earth/jvegas/pyCharm/cfutools/interpolation/scrip_use '
                                        '{0}'.format(namelist_file), Log.NO_LOG)
            os.remove(namelist_file)
            nco.ncecat(input=temp2, output=temp2, options="-O -h")
            shutil.move(temp2, 'tmp_{0:02d}.nc'.format(lev+1))
            Log.debug("Level {0} ready", lev)

    @staticmethod
    def cut_section(input_file, output_file, variable, zonal, value):
        """
        Cut a meridional or zonal section

        Created in September 2012     Author : vguemas@ic3.cat
        :param input_file: input file
        :param output_file: output file  ( => 2D )
        :param variable: input var
        :param zonal: (zonal / meridional section)
        :param value: lat/lon
        :return:
        """

        nco = Utils.nco

        handler = nco.openCdf('mesh_hgr.nc')
        dimi = handler.dimensions['i'].size
        dimj = handler.dimensions['j'].size
        dimlev = handler.dimensions['lev'].size

        lon = handler.variables['lon'][:]
        lat = handler.variables['lat'][:]
        handler.close()

        handler = nco.openCdf('mask.nc')
        mask_lev = handler.variables['tmask'][:]
        mask_lev = mask_lev.astype(float)
        # noinspection PyTypeChecker
        np.place(mask_lev, mask_lev == 0, [1e20])
        handler.close()

        # Latitude / longitude of the zonal / meridional section
        exactpos = value
        if not zonal:
            while exactpos < np.min(lon):
                exactpos += 360
            while exactpos > np.max(lon):
                exactpos -= 360
            size = dimj
        else:
            size = dimi

        # Collect the indexes defining the section

        listi = np.empty(size, dtype=int)
        listj = np.empty(size, dtype=int)

        for jpt in range(0, size):
            if not zonal:
                vector = lon[jpt, :]
            else:
                vector = lat[:, jpt]
            distance = abs(vector - exactpos)
            pos = np.where(distance == min(distance))
            if not zonal:
                listi[jpt] = pos[0][0]
                listj[jpt] = jpt
            else:
                listi[jpt] = jpt
                listj[jpt] = pos[0][0]

        temp = TempFile.get()
        shutil.copy(input_file, temp)

        handler = nco.openCdf(temp)
        dimtime = handler.dimensions['time'].size
        var_array = handler.variables[variable][:]
        handler.close()

        var = np.empty([dimtime, dimlev, size], dtype=handler.variables[variable].dtype)
        new_coord = np.empty(size, dtype=float)
        if zonal:
            old_coord = lon
        else:
            old_coord = lat

        for jpt in range(0, size):
            Log.info(str(jpt))
            var[:, :, jpt] = np.maximum(var_array[:, :, listj[jpt], listi[jpt]],
                                        mask_lev[:, :, listj[jpt], listi[jpt]])
            new_coord[jpt] = old_coord[listj[jpt], listi[jpt]]

        nco.ncks(input=temp, output=temp, options='-O -v lev,time')

        handler = nco.openCdf(temp)
        if not zonal:
            handler.createDimension('lat', size)
            coord_var = handler.createVariable('lat', float, 'lat')
            file_var = handler.createVariable(variable, float, ('time', 'lev', 'lat'))
        else:
            handler.createDimension('lon', size)
            coord_var = handler.createVariable('lon', float, 'lon')
            file_var = handler.createVariable(variable, float, ('time', 'lev', 'lon'))
        coord_var[:] = new_coord[:]
        file_var[:] = var[:]
        file_var.missing_value = 1e20
        handler.close()
        Utils.move_file(temp, output_file)

    @staticmethod
    def avgsection(input_file, output_file, lon_min, lon_max, lat_min, lat_max):
        temp = TempFile.get()
        Utils.cdo.zonmean(input='-sellonlatbox,{0},{1},{2},{3} {4}'.format(lon_min, lon_max, lat_min, lat_max,
                                                                           input_file),
                          output=temp)
        Utils.move_file(temp, output_file)


