from earthdiagnostics import Utils, cdftools, cdo, TempFile
from earthdiagnostics.basins import Basins
from autosubmit.config.log import Log
import os
import shutil
import traceback
import numpy as np
from nco import NCOException


class Heat(object):
    """
    Diagnsotics for the ocean heat content
    """

    @staticmethod
    def mixed_layer_content(input_file, mltost_file, output_file):
        """
        Compute mixed layer heat and salt content

        Created in February 2012      Author : vguemas@ic3.cat

        :param mltost_file: imput mltost file
        :param input_file: input grid_T file name
        :param output_file: output file name  (=> 2D x-y )
        :return:
        """

        nco = Utils.nco

        input_scratch = TempFile.get()
        shutil.copy(input_file, input_scratch)
        nco.ncks(input=mltost_file, output=input_scratch, options='-A -v mlotst')

        ntime = int(cdo.ntime(input=input_scratch)[0])
        files = list()

        for time in range(ntime):
            Log.info('Running time {0}', time)
            temp = TempFile.get()
            nco.ncks(input=input_scratch, output=temp, options='-O -d time,{0}'.format(time))
            Log.info('Computing salt content')
            cdftools.run('cdfmxlheatc', input=temp, options=['-full'])
            Utils.move_file('mxlheatc.nc', temp)
            files.append(temp)

        temp = TempFile.get()
        cdo.cat(input=' '.join(files), output=temp,)
        nco.ncks(input=input_scratch, output=temp, options='-A -v time')

        for temp_file in files:
            os.remove(temp_file)
        os.remove(input_scratch)

        Utils.rename_variables(temp, {'x': 'i', 'y': 'j', 'somxlheatc': 'mlotsthc'}, False, True)
        Utils.setminmax(temp, 'mlotsthc')
        Utils.move_file(temp, output_file)

    @staticmethod
    def layer(input_file, output_file, min_depth, max_depth):
        """
        Pointwise Ocean Heat Content in a specified ocean thickness (J/m-2)

        Created in June 2012      Author : isabel.andreu-burillo@ic3.cat
        May 2014 - Virginie Guemas - Way around the bc that does not work on moore

        :param input_file: input grid_T file nam
        :param output_file: output file name  (=> 2D x-y )
        :param min_depth: upper depth of the layer (in meters)
        :param max_depth: lower depth of the layer (in meters)
        :return:
        """

        nco = Utils.nco

        temp = TempFile.get()
        heatc_sl_out = TempFile.get()
        heatc_sl_top = TempFile.get()
        level_above = TempFile.get()
        level_below = TempFile.get()
        heatc_sl_bottom = TempFile.get()
        heatc_sl_top_invert = TempFile.get()
        e3tfile = TempFile.get()
        results = TempFile.get()

        nco.ncap2(input='mesh_zgr.nc', output=e3tfile, options='-v -O -s "heatc_sl=tmask*e3t"'),

        shutil.copy(input_file, temp)

        Utils.rename_variable(temp, 'thetao', 'heatc_sl')

        nco.ncks(input=temp, output=temp, options='-O -v heatc_sl')
        cdo.mul(input=' '.join([temp, e3tfile]), output=heatc_sl_out)

        # extract the data between the two given depths --> heatc_sl_top.nc
        nco.ncks(input=heatc_sl_out, output=heatc_sl_top,
                 options='-O -d lev,{0}.0,{1}.0'.format(min_depth, max_depth))

        # now extract a few levels below, to compute the residual ohc
        # addition with float returned:
        nco.ncks(input=heatc_sl_out, output=heatc_sl_bottom,
                 options='-O -d lev,{0}.0,{1}.0'.format(max_depth, max_depth+200))

        # obtain the weight for the extra level containing the 300 m
        # deptht in the gridT files is positive
        # weight = (300.0 - depth_top)/(depth_bottom - depth_top)
        # and add the thickness down to 300 m in the next layer
        nco.ncpdq(options="-a '-lev'", input=heatc_sl_top, output=heatc_sl_top_invert)
        nco.ncks(input=heatc_sl_top_invert, output=level_above, options='-O -d lev,0,0,1')
        nco.ncks(input=heatc_sl_bottom, output=level_below, options='-O -d lev,0,0,1')

        handler = nco.openCdf(level_above)
        lev_above = handler.variables['lev'][:]
        handler.close()

        handler = nco.openCdf(level_below)
        layerthcknss = handler.variables['lev'][:] - lev_above
        heatc_sl_below = handler.variables['heatc_sl'][:]
        handler.close()

        factor = (max_depth - lev_above) / layerthcknss

        heatc_sl_below = heatc_sl_below * factor
        handler = nco.openCdf(heatc_sl_top)
        heatc_sl = handler.variables['heatc_sl'][:]
        handler.close()

        heatc_sl = np.sum(heatc_sl, 1)
        heatc_sl = heatc_sl[:] + heatc_sl_below[:, 0, :]
        heatc_sl = heatc_sl[:] * 1020 * 4000

        nco.ncks(input=heatc_sl_top_invert, output=results, options='-O -v lon,lat,time')
        handler_results = nco.openCdf(results)
        handler_results.createVariable('ohc', float, ('time', 'y', 'x'))
        handler_results.close()

        handler_results = nco.openCdf(results)
        handler_results.variables['ohc'][:] = heatc_sl
        handler_results.close()

        Utils.setminmax(results, 'ohc')
        Utils.move_file(results, output_file)

    @staticmethod
    def total(input_file, mlotst_file, output_file, basin=Basins.Global.shortname, mixed_layer=0,
              upper=None, lower=None):
        """
        Compute the total ocean heat extent

        Created in May 2012           Author : vguemas@ic3.cat

        :param mlotst_file:
        :param input_file: input temperature file name
        :param output_file: output file name  ( => 2D x-y
        :param basin: basin name (default: global)
        :param mixed_layer: mixed layer (1=only, 0=included, -1=without)
        :param upper: upper level of the layer (optional) Default : top
        :param lower: lower level of the layer (optional) Default : bottom
        """

        nco = Utils.nco
        temp = TempFile.get()
        temp2 = TempFile.get()

        shutil.copy(input_file, temp)
        nco.ncks(input=mlotst_file, output=temp, options='-A -v mlotst')

        basin = Basins.parse(basin)

        handler = nco.openCdf('mask_regions.3d.nc')
        if basin.fullname not in handler.variables:
            raise Exception('Basin {0} is not defined on mask_regions.nc'.format(basin.fullname))

        handler.close()
        if basin != Basins.Global:
            shutil.move('mask.nc', 'original_mask.nc')
            shutil.move('mask_regions.3d.nc', 'mask.nc')
            Utils.rename_variable('mask.nc', basin.fullname, 'tmask')
        error = None

        try:
            para = list()
            para.append("-full")
            para.append(str(basin.min_lon))
            para.append(str(basin.max_lon))
            para.append(str(basin.min_lat))
            para.append(str(basin.max_lat))
            para.append(upper)
            para.append(lower)
            para.append(str(mixed_layer))
            shell_output = cdftools.run('cdfheatc', options=para, input=temp)
        except Exception as ex:
            error = ex.message
        finally:
            if basin != Basins.Global:
                Utils.rename_variable('mask.nc', 'tmask', basin.fullname)
                shutil.move('mask.nc', 'mask_regions.3d.nc')
                shutil.move('original_mask.nc', 'mask.nc')
            if error:
                raise Exception(error)

        nco.ncks(input=temp, output=temp2, options='-O -v time')
        output = nco.openCdf(temp2)

        thc = output.createVariable('thc', float, 'time')
        thc.standard_name = "integral_of_sea_water_potential_temperature_expressed_as_heat_content"
        thc.long_name = "Total heat content"
        thc.units = "Joules"

        uhc = output.createVariable('uhc', float, 'time')
        uhc.standard_name = "integral_of_sea_water_potential_temperature_expressed_as_heat_content"
        uhc.long_name = "Heat content per unit volume"
        uhc.units = "Joules/m3"
        time = 0
        # noinspection PyUnboundLocalVariable
        for lines in shell_output:
            if not lines:
                continue

            for line in lines.split('\n'):
                line = line.lstrip()
                if line.startswith("Heat Content  at level"):
                    Log.info(line)
                elif line.startswith("Total Heat content/volume"):
                    Log.user_warning(line)
                    uhc[time] = line[line.index(':')+1: line.index('Joules')]
                    time += 1
                if line.startswith("Total Heat content "):
                    Log.result(line)
                    thc[time] = line[line.index(':')+1: line.index('Joules')]
                elif line.startswith('TIME : '):
                    Log.info(line)

        output.close()
        Utils.setminmax(temp2, ['thc', 'uhc'])
        Utils.move_file(temp2, output_file)


def main():
    try:
        Heat.layer("ORCA1_MM_19601101_19610228_grid_T.nc", "out.nc", 300, 2000)
    except NCOException as ex:
        Log.error('Exception {0}: stderr: {1}, stdout: {2}',  ex.msg, ex.stderr, ex.stdout)
        Log.error(traceback.format_exc())


if __name__ == "__main__":
    main()
