from earthdiagnostics import Utils, cdftools, TempFile
from autosubmit.config.log import Log
import os
import shutil


class Salinity(object):
    """
    Class containing all the diagnostics related to salinity
    """

    @staticmethod
    def vertical_mean(input_file, output_file, upper, lower):
        """
        Vertically averaged salt content
        Created in February 2012      Author : vguemas@ic3.cat

        :param input_file: input grid_T file name
        :param upper: upper depth of the layer (in meters)
        :type upper: int
        :param lower: lower depth of the layer (in meters)
        :type lower: int
        :param output_file: output file name  (=> 2D)
        """
        cdo = Utils.cdo
        nco = Utils.nco

        temp = TempFile.get()
        temp2 = TempFile.get()
        Log.debug('Calculating vertical mean between {0} and {1}', upper, lower)
        Log.debug('Input file: {0}', input_file)
        Log.debug('Output file: {0}', output_file)
        # test on cdo version: if >1.5.6, remove valid_min/max attributes to avoid values
        # out of that range to be replaced by NaN
        #
        if cdo.version() > u'1.5.6':
            nco.ncatted(input=input_file, output=temp, options='-O -a valid_max,lev,d,,')
            nco.ncatted(input=temp, output=temp, options='-O -a valid_min,lev,d,,')
        else:
            shutil.copy(input_file, temp)

        cdftools.run('cdfvertmean', input=temp, output=temp2, options=['so', 'T', str(upper), str(lower),
                                                                       '-full', '-debug'])
        os.remove(temp)
        Utils.setminmax(temp2, 'so_vert_mean')
        Utils.rename_variable(temp2, 'so_vert_mean', 'someanm{0}-{1}'.format(upper, lower))
        Utils.move_file(temp2, output_file)
        Log.debug('Finished!')

    @staticmethod
    def mixed_layer_content(input_file, mlotst_file, output_file):
        """
        Compute mixed layer heat and salt content

        Created in February 2012      Author : vguemas@ic3.cat

        :param mlotst_file:
        :param input_file: input grid_T file name
        :param output_file: output file name  (=> 2D x-y )
        :return:
        """
        nco = Utils.nco
        cdo = Utils.cdo
        input_scratch = TempFile.get()
        shutil.copy(input_file, input_scratch)
        Utils.nco.ncks(input=mlotst_file, output=input_scratch, options='-A -v mlotst')

        ntime = int(cdo.ntime(input=input_scratch)[0])
        files = list()

        for time in range(ntime):
            Log.info('Running time {0}', time)
            temp = TempFile.get()
            nco.ncks(input=input_scratch, output=temp, options='-O -d time,{0}'.format(time))
            Log.info('Computing heat content')
            cdftools.run('cdfmxlsaltc', input=temp, options=['-full'])
            Utils.move_file('mxlsaltc.nc', temp)
            files.append(temp)

        temp = TempFile.get()
        cdo.cat(input=' '.join(files), output=temp,)
        nco.ncks(input=input_scratch, output=temp, options='-A -v time')

        for temp_file in files:
            os.remove(temp_file)
        os.remove(input_scratch)

        Utils.rename_variables(temp, {'x': 'i', 'y': 'j', 'somxlsaltc': 'mlotstsc'}, False, True)
        Utils.setminmax(temp, 'mlotstsc')
        Utils.move_file(temp, output_file)
